import type { BaseFieldItem, TableFields } from '@/types/Tables/table'
import type { Ref } from 'vue'

export function useCustomIndicatorDialog() {
  /**
   * @description: 初始化一下被排序过的表格字段信息，这里要注意深拷贝
   * @return {*}
   */
  const initSortedTableFieldsInfo = (
    sortedTableFieldsInfo: Array<TableFields>,
    propsTableFieldsInfo: Array<TableFields>,
  ) => {
    sortedTableFieldsInfo.splice(
      0,
      sortedTableFieldsInfo.length,
      ...JSON.parse(JSON.stringify(propsTableFieldsInfo)),
    )
  }

  /**
   * @description: 验证自定义指标名称
   * @param {*} _rule 规则
   * @param {*} value 值
   * @param {*} callback 回调
   * @return {*}
   */
  const validCustomName = (_rule: any, value: any, callback: any) => {
    const regex = new RegExp(
      `^[\u4e00-\u9fa5a-zA-Z][\u4e00-\u9fa5_a-zA-Z0-9]{0,9}$`,
    )
    if (regex.test(value)) {
      callback()
    } else {
      callback(
        new Error('请输入1-10位中文、英文、数字或下划线，必须以中英文字符开头'),
      )
    }
  }

  /**
   * @description: 更新固定指标的框的值
   * @return {*}
   */
  const updateFixedIndicator = (
    propIndicator: Array<BaseFieldItem<TableFields>>,
    fixedIndicator: TableFields[],
  ) => {
    let result: TableFields[] = []
    propIndicator.forEach(item => {
      result.push(
        ...item.children.filter(item => {
          return item.fixed
        }),
      )
    })
    fixedIndicator.splice(0, fixedIndicator.length, ...result)
  }

  /**
   * @description: 同步右侧栏目和表格字段的顺序
   * @param {Array<TableFields>} propsTableFields 表格字段
   * @param {Array<string>} allSelected 所有目前选中的字段名
   * @return {*}
   */
  const syncTableField = (
    propsTableFields: Array<TableFields>,
    allSelected: Array<string>,
  ) => {
    let syncFieldSelected: Array<string> = []
    propsTableFields.forEach(item => {
      let fieldName = allSelected.find(select => select === item.name)
      if (fieldName) syncFieldSelected.push(fieldName)
    })
    allSelected.splice(0, allSelected.length, ...syncFieldSelected)
  }

  /**
   * @description: 更新不固定框的状态和顺序
   * @return {*}
   */
  const updateUnfixedIndicator = (
    propIndicator: Array<BaseFieldItem<TableFields>>,
    isFirstShow: Ref<boolean>,
    propsTableFields: Array<TableFields>,
    unFixedIndicator: TableFields[],
  ) => {
    let result: TableFields[] = []

    // 拿到所有的子元素
    let allChilds = propIndicator.flatMap(
      item => item.children,
    ) as Array<TableFields>

    // 重置所有状态
    allChilds.forEach(item => {
      if (!item.fixed) item.state = false
    })

    // 获取所有选中的值
    let allSelected = propIndicator.flatMap(item => item.value) as Array<string>

    // 在打开对话框的时候要去同步一下表格目前显示的字段和对话框右侧栏目的顺序
    if (isFirstShow.value && propsTableFields.length > 0) {
      syncTableField(propsTableFields, allSelected)
      isFirstShow.value = false // 只需要第一次的时候同步一下
    }

    // 之所以要这样去筛选，是为了保证顺序，
    //  如果直接通过filter去找存在于allSelected中的元素，
    //  那么当一个在前面的选项首先取消选择，然后又重新选中的时候，这个选项又会直接出现在他原来的位置，而不是末尾
    allSelected.forEach(item => {
      let child = allChilds.find(child => !child.fixed && child.name === item)
      if (child) {
        child.state = true // 更新状态
        result.push(child)
      }
    })
    unFixedIndicator.splice(0, unFixedIndicator.length, ...result)
  }

  /**
   * @description: 更新一下排序后的表格字段信息
   * @return {*}
   */
  const updateSortedTableFieldsInfo = (
    sortedTableFieldsInfo: Array<TableFields>,
    dragContentRef: Ref<HTMLElement | null>,

    unFixedIndicator: TableFields[],
    fixedIndicator: TableFields[],
  ) => {
    sortedTableFieldsInfo.splice(0, sortedTableFieldsInfo.length)
    let sortedNodes = Array.from(dragContentRef.value!.children)
    let sortedFields: Array<string> = []
    // 把所有能够排序的节点的name提取出来，这些就是已经排好序的字段名
    sortedNodes.forEach((node: Element) => {
      if (node.nodeType === 1) {
        // 判断是否为元素节点
        sortedFields.push(node.getAttribute('name') as string)
      }
    })

    // 根据排好序的字段名，去原本的字段信息中去找对应的字段名，把他加入到新字段信息中
    // 这里的排序与unFixedIndicator的排序不一样，这里是为了同步用户的拖动
    // unFixedIndicator的排序是为了同步新增的选项
    sortedFields.forEach((item: string) => {
      let fFiled = unFixedIndicator.find(field => field.name === item)
      if (fFiled) sortedTableFieldsInfo.push(fFiled)
    })
    // 还剩下固定字段的信息，把他加入数组头部
    sortedTableFieldsInfo.unshift(...fixedIndicator)
  }

  /**
   * @description: 获取当前被激活的指标
   * @return {*}
   */
  const getActivedIndicator = (
    propIndicator: Array<BaseFieldItem<TableFields>>,
  ): string[] => {
    let result: string[] = []
    propIndicator.forEach(item => {
      item.children.forEach(field => {
        if (field.state) result.push(field.name)
      })
    })
    return result
  }

  return {
    initSortedTableFieldsInfo,
    validCustomName,
    updateFixedIndicator,
    syncTableField,
    updateUnfixedIndicator,
    getActivedIndicator,
    updateSortedTableFieldsInfo,
  }
}
