/*
 * @Author: fxs bjnsfxs@163.com
 * @Date: 2024-10-25 15:26:00
 * @LastEditors: fxs bjnsfxs@163.com
 * @LastEditTime: 2024-11-02 09:52:13
 * @FilePath: \Quantity-Creation-Management-System\src\hooks\useDialogDrag.ts
 * @Description: 只用于自定义指标对话框的拖拽功能
 *
 */

import type { Ref } from 'vue'

export function useDialogDrag(
  dragingRef: Ref<HTMLElement | null>,
  dragContentRef: Ref<HTMLElement | null>,
) {
  /**
   * @description: 判断当前是否可以插入
   * @param {*} e 触发事件的事件对象
   * @return {*}
   */
  const canInsert = (e: DragEvent): boolean => {
    return !(
      !e.target ||
      !dragingRef.value ||
      !dragContentRef.value ||
      dragingRef.value === e.target ||
      e.target === dragContentRef.value
    )
  }

  /**
   * @description: 拖动开始
   * @param {*} e 拖动事件对象
   * @return {*}
   */
  const dragStart = (e: DragEvent) => {
    if (!dragContentRef.value || !e.target || !e.dataTransfer) return
    e.dataTransfer.effectAllowed = 'move' // 设置拖拽类型
    dragingRef.value = e.target as HTMLElement // 获取当前拖拽的元素
    dragingRef.value.classList.add('dragMove') // 添加拖拽样式
  }

  /**
   * @description: 进入拖拽元素触发
   * @param {*} e 拖拽事件对象
   * @return {*}
   */
  const dragEnter = (e: DragEvent) => {
    e.preventDefault()
    if (!canInsert(e)) return
    let listArray = Array.from(dragContentRef.value!.childNodes) // 整个可拖动的列表
    let curIndex = listArray.indexOf(dragingRef.value!) // 当前拖动元素在整个列表中的文职
    let targeIndex = listArray.indexOf(e.target as HTMLElement) // 目前拖动到的元素在整个列表中的位置

    if (curIndex < targeIndex) {
      dragContentRef.value!.insertBefore(
        dragingRef.value!,
        (e.target as HTMLElement).nextSibling,
      )
    } else {
      dragContentRef.value!.insertBefore(
        dragingRef.value!,
        e.target as HTMLElement,
      )
    }
  }

  /**
   * @description: 在目标元素上拖动
   * @param {*} e 拖拽对象
   * @return {*}
   */
  const dragOver = (e: DragEvent) => {
    e.preventDefault()
    if (!e.target) return
  }

  /**
   * @description: 拖拽结束
   * @param {DragEvent} e 拖拽对象
   * @return {*}
   */
  const dragEnd = (e: DragEvent) => {
    if (!e.target) return
    dragingRef.value?.classList.remove('dragMove')
  }

  return {
    dragStart,
    dragEnter,
    dragOver,
    dragEnd,
  }
}
