import type {
  BaseFieldItem,
  CustomIndicatorScheme,
  TableData,
  TableFields,
} from '@/types/Tables/table'
import type { BaseTableInfo } from '@/types/Tables/tablePageData'
import type { ComputedRef, Reactive, Ref } from 'vue'
import type { TablePaginationProps } from '@/types/Tables/pagination'
import type { BaseMenu } from '@/types/Promotion/Menu'
import type { MenuTableReq } from '@/types/Tables/MenuTable/menuTableReq'

import { useTable } from './useTable'
import Table from '@/components/table/Table.vue'
import TableCustomIndicatorController from '@/utils/localStorage/tableCustomIndicatorController'

type TableType = InstanceType<typeof Table>

export function useMenuTable(
  saveName: string, // 保存的文件名
  fileName: ComputedRef<string>, // 动态的文件名
  activeMenu: Ref<string>, // 当前激活的菜单
  customIndicatorScheme: Ref<Array<CustomIndicatorScheme>>, // 自定义指标方案
  // 表格IO控制器
  tableControllers: {
    [key: string]: TableCustomIndicatorController
  },
  tableInfo: Reactive<BaseTableInfo>, // 表格信息
  tableReqInfo: MenuTableReq, // 表格请求信息
  tablePaginationConfig: Reactive<TablePaginationProps>, // 表格分页配置
  tableRef: Ref<TableType | null>, // 表格实例
  menuList: BaseMenu[], // 菜单列表
) {
  const { tableReqList, getData, updateTableFields } = useTable()

  /**
   * @description: 更新表格数据
   * @return {*}
   */
  const updateTableData = async () => {
    try {
      tableRef.value?.changeTableLoading(true)
      updateTableInfo()
      if (tableRef.value) {
        tableReqInfo[activeMenu.value].config.params =
          tableRef.value?.getTableReqParams()
      }

      await getData(tableReqInfo[activeMenu.value]).then(
        (res: Array<TableData>) => {
          tableInfo[activeMenu.value].data.splice(
            0,
            tableInfo[activeMenu.value].data.length,
            ...res,
          )
          tablePaginationConfig.total = res.length
        },
      )
    } catch (err) {
      ElMessage.error('获取数据失败')
      console.log(err)
    } finally {
      // 只有当表格中没有请求的时候，再去把加载状态关闭
      if (tableReqList.length === 0) tableRef.value?.changeTableLoading(false)
    }
  }

  /**
   * @description: 切换菜单
   * @param {*} val 新菜单名
   * @return {*}
   */
  const menuActiveChange = (val: string) => {
    activeMenu.value = val
    customIndicatorScheme.value =
      tableControllers[fileName.value].getSchemeList()
    // updateTableData()
  }

  /**
   * @description: 更新表格的字段信息
   * @return {*}
   */
  const updateTableInfo = (customName: string = '默认') => {
    if (!customName) customName = '默认'
    let info = tableControllers[fileName.value].getCustomScheme(customName)

    if (info) {
      tableInfo[activeMenu.value].fields = info.fields
      tableInfo[activeMenu.value].tableSortedFields = info.tableSortedFields
    }
  }

  /**
   * @description: 更新自定义指标和表格字段
   * @param {*} newIndicator  新的指标信息
   * @param {*} newSortedTablefileds 新的表格字段信息
   * @return {*}
   */
  const updateCustomIndicator = (
    newIndicator: Array<BaseFieldItem<TableFields>>,
    newSortedTablefileds: Array<TableFields>,
    customSaveName: string,
  ) => {
    updateTableFields(
      tableInfo,
      activeMenu.value,
      newIndicator,
      newSortedTablefileds,
    )
      .then(() => {
        tableControllers[fileName.value].saveCustomScheme(
          JSON.parse(JSON.stringify(tableInfo[activeMenu.value])),
          customSaveName,
        )

        customIndicatorScheme.value =
          tableControllers[fileName.value].getSchemeList()
      })
      .catch(() => {
        ElMessage.error('更新失败')
      })
    // saveTableInfo()
  }

  /**
   * @description: 改变方案，要去更新一下表格字段
   * @param {*} schemeName 新的方案名
   * @return {*}
   */
  const changeScheme = (schemeName: string) => {
    updateTableInfo(schemeName)
    tableRef.value?.updateIndicatorScheme()
  }

  /**
   * @description: 初始化表格的保存和读取文件的控制器
   * @return {*}
   */
  const initTableController = () => {
    menuList.forEach(item => {
      let tableName = `${saveName}-${item.name}`
      tableControllers[tableName] = new TableCustomIndicatorController(
        tableName,
        tableInfo[item.name],
      )
    })
  }

  /**
   * @description: 初始化数据
   * @return {*}
   */
  const initData = () => {
    initTableController()
    updateTableData()
    customIndicatorScheme.value =
      tableControllers[fileName.value].getSchemeList()
  }

  return {
    updateTableData,
    menuActiveChange,
    updateTableInfo,
    changeScheme,
    updateCustomIndicator,
    initTableController,
    initData,
  }
}
