import type { TableData, BaseFieldInfo } from '@/types/Tables/table'
import type { BaseTableInfo } from '@/types/Tables/tablePageData'
import type { ResponseInfo } from '@/types/axios'
import type { BaseFieldItem, TableFields } from '@/types/Tables/table'
import type { Ref } from 'vue'
import type { PaginationConfig } from '@/types/Tables/pagination'
import type { TableReq } from '@/types/Tables/MenuTable/menuTableReq'

import { resetReactive } from '@/utils/common'
import { writeFileXLSX, utils } from 'xlsx'

import axiosInstance from '@/utils/axios/axiosInstance'

export function useTable() {
  let controller: AbortController | null = null // 请求控制器，用于取消请求
  let tableReqList: Array<string> = [] // 表格的请求列表，暂时只存了url，可以存其他的

  /**
   * @description: 获取表格数据
   * @param {string} url 请求地址
   * @param {Object} config 配置
   * @return {*}
   */
  const getData = async (reqInfo: TableReq): Promise<Array<TableData>> => {
    try {
      let { url, config } = reqInfo
      tableReqList.push(url) // 添加请求
      controller && controller.abort() // 取消上一次请求
      controller = new AbortController()

      config.signal = controller.signal
      const res = (await axiosInstance.get(url, config)) as ResponseInfo

      if (res.code !== 0) throw new Error('请求失败')
      return res.data as TableData[]
    } catch (err) {
      console.log('数据请求失败')
      console.log(err)
      return []
    } finally {
      tableReqList.pop() // 请求完成后移除一个请求
    }
  }

  /**
   * @description: 判断当前字段是否是需要固定的字段
   * @param {*} info 字段信息
   * @return {boolean}
   */
  const isFixedField = (
    fixedFields: Array<string>,
    info: BaseFieldInfo,
  ): boolean => {
    return fixedFields.includes(info.name)
  }

  /**
   * @description: 更新自定义指标和表格字段
   * @param {*} newIndicator  新的指标信息
   * @param {*} newSortedTablefileds 新的表格字段信息
   * @return {*}
   */
  const updateTableFields = (
    tableInfo: BaseTableInfo,
    activeName: string,
    newTableFiles: Array<BaseFieldItem<TableFields>>,
    newSortedTablefileds: Array<TableFields>,
  ): Promise<boolean> => {
    return new Promise((reslove, reject) => {
      try {
        let oldTablefields = tableInfo[activeName].fields
        let oldSortedFields = tableInfo[activeName].tableSortedFields

        oldTablefields.splice(0, oldTablefields.length, ...newTableFiles)

        oldSortedFields.splice(
          0,
          oldSortedFields.length,
          ...newSortedTablefileds,
        )
        reslove(true)
      } catch (err) {
        console.log('更新失败')
        console.log(err)
        reject(false)
      }
    })
  }

  /**
   * @description: 初始化自定义指标的字段信息
   * @return {*}
   */
  const initIndicatorFields = (
    indicatorFields: Array<BaseFieldItem<TableFields>>,
    propsTableFields: Array<BaseFieldItem<TableFields>>,
    defaultActiveNav: Ref<string>,
  ) => {
    resetReactive(indicatorFields)
    indicatorFields.splice(
      0,
      indicatorFields.length,
      ...JSON.parse(JSON.stringify(propsTableFields)),
    )
    defaultActiveNav.value = indicatorFields[0].name
  }

  /**
   * @description: 初始化表格字段，把所有字段都放到一个数组中
   * @return {*}
   */
  const initTableFields = (
    tableFieldsInfo: Array<TableFields>,
    sortedTableFields: Array<TableFields>,
    indicatorFields: Array<BaseFieldItem<TableFields>>,
  ) => {
    resetReactive(tableFieldsInfo)
    // 如果传入了排序字段，则直接使用排序字段，说明已经有缓存了
    if (sortedTableFields.length > 0) {
      tableFieldsInfo.splice(0, tableFieldsInfo.length, ...sortedTableFields)
    } else {
      // 没传入排序字段，则根据固定字段和自定义指标字段进行排序
      indicatorFields.forEach(item => {
        item.children.forEach(child => {
          if (child.fixed || child.state) {
            item.value.push(child.name)
            child.state = true
          }
          tableFieldsInfo.push(JSON.parse(JSON.stringify(child)))
        })
      })
    }
  }

  /**
   * @description: 缓存表格数据。只有是远程查询的时候才会开启缓存
   * @param {*} newData 新的表格数据
   * @return {*}
   */
  const setCacheTableData = (
    isRemote: boolean,
    paginationConfig: PaginationConfig,
    cacheTableData: Array<Array<TableData>>,
    newData: Array<TableData>,
  ) => {
    if (isRemote) {
      // 这页没有数据，就缓存一下
      let curPage = paginationConfig.curPage
      let curPageSize = paginationConfig.curPageSize
      if (!cacheTableData[curPage] || cacheTableData[curPage].length === 0) {
        cacheTableData[curPage] = newData.splice(0, curPageSize) // 缓存数据，后面的splice是防止返回的数据超出当前页长度
      }
    }
  }
  /**
   * @description: 导出表格数据位Excel未见
   * @param {Array} propTablefields 表格字段信息
   * @param {Array<any>} tableData 表格数据
   * @param {string} fileName 保存的文件名
   * @param {Array<string>} excludeExportFields 被排除的字段
   * @return {*}
   */
  const exportDataToExcel = (
    propTablefields: Array<BaseFieldItem<TableFields>>,
    tableData: Array<any>,
    fileName: string,
    excludeExportFields: Array<string>,
  ) => {
    try {
      // 构建导出的表格数据
      const selectedData = tableData.slice(0, tableData.length)

      // 表头，这里重新生成一下
      // 这种形式：{name:'姓名', age:'年龄'}
      let header: {
        [key: string]: string
      } = {}
      propTablefields.forEach(item => {
        item.children.forEach(child => {
          if (!excludeExportFields.includes(child.name))
            header[child.name] = child.label
        })
      })

      // 拼接数据
      const exportData = [header, ...selectedData]

      // 转为sheet对象，skipheader要选上，因为上面拼了一个表头
      const ws = utils.json_to_sheet(exportData, { skipHeader: true })

      // 创建工作部并且附加到一个工作表中，这个工作表名可以自定义
      const wb = utils.book_new()
      utils.book_append_sheet(wb, ws, fileName)
      // 导出
      writeFileXLSX(wb, fileName)
    } catch (err) {
      console.log(err)
      ElMessage.error('导出失败')
    }
  }

  return {
    tableReqList,
    getData,

    isFixedField,
    exportDataToExcel,
    updateTableFields,
    initTableFields,
    initIndicatorFields,
    setCacheTableData,
  }
}
