/*
 * @Author: fxs bjnsfxs@163.com
 * @Date: 2024-08-26 15:46:42
 * @LastEditors: fxs bjnsfxs@163.com
 * @LastEditTime: 2024-11-11
 * @FilePath: \Quantity-Creation-Management-System\src\utils\common\index.ts
 * @Description:
 *
 */

import { ElMessage } from 'element-plus'
import type { Reactive } from 'vue'

/**
 * @description:  包装防抖函数
 * @param {array} func 函数
 * @param {number} delay 时间
 * @return {*}
 */
export function debounceFunc<T extends (...args: any[]) => any>(
  func: T,
  delay: number,
): (...args: Parameters<T>) => void {
  let timer: ReturnType<typeof setTimeout> | null
  return (...args: Parameters<T>) => {
    if (timer) clearTimeout(timer)
    timer = setTimeout(() => {
      func(...args)
    }, delay)
  }
}

/**
 * @description: 包装节流函数
 * @param {T} func 函数
 * @param {number} delay 时间
 * @return {*}
 */
export function throttleFunc<T extends (...args: any[]) => any>(
  func: T,
  delay: number,
): (...args: Parameters<T>) => void {
  let lastCall = 0

  return function (...args: Parameters<T>) {
    const now = Date.now()

    if (now - lastCall >= delay) {
      lastCall = now
      func(...args)
    }
  }
}

// 小数转百分比
export function decimalToPercentage(
  decimal: number,
  decimalPlaces: number = 0,
): string {
  // Convert the decimal to a percentage by multiplying by 100
  const percentage = decimal * 100

  // Format the percentage to a fixed number of decimal places
  const formattedPercentage = percentage.toFixed(decimalPlaces)

  // Append the '%' symbol and return the result
  return `${formattedPercentage}%`
}

// 生成时间刻度
export function generateHourlyArray(count: number) {
  const result = []
  for (let i = 0; i <= count; i++) {
    result.push(`${i}:00`)
  }
  return result
}

/**
 * @description: 格式化一个时间为year-month-day的格式
 * @param {Date} date 日期
 * @return {string} 返回格式化后的字符串
 */
export function formatterDate(date: Date): string {
  date.setHours(0, 0, 0, 0)
  // 格式化日期为 'YYYY-MM-DD' 格式
  const year = date.getFullYear()
  const month = String(date.getMonth() + 1).padStart(2, '0') // 月份从0开始，需要加1
  const day = String(date.getDate()).padStart(2, '0')

  // 返回格式化的字符串
  return `${year}-${month}-${day}`
}

/**
 * @description: 将日期或日期数组转为year-month-day格式
 * @param {Date} dateTime 日期或日期数组
 * @return {string | string[]} 返回格式化的日期或日期数组
 */
export function batchFormatterTime<T extends Date | Date[]>(
  dateTime: T,
): T extends Date[] ? string[] : string {
  if (Array.isArray(dateTime)) {
    return dateTime.map(item => formatterDate(item)) as any
  }
  // 返回格式化的字符串
  return formatterDate(dateTime) as any
}

/**
 * @description:  判断日期范围是否有效
 * @param {string} startTime 开始时间
 * @param {string} endTime 结束时间
 * @return {boolean} 返回是否有效
 */
export const invaildDateRange = (
  startTime: string,
  endTime: string,
): boolean => {
  const start = new Date(startTime).getTime()
  const end = new Date(endTime).getTime()
  // 判断日期是否有效，`isNaN` 判断生成的时间是否为 "Invalid Date"
  return !isNaN(start) && !isNaN(end) && start < end
}

/**
 * @description: 生成一个打包后可以使用的url
 * @param {string} url  传入一个assets文件夹下的文件名
 * @return {*}
 */
export const getAssetsImageUrl = (url: string) => {
  return new URL(`../../assets/${url}`, import.meta.url).href
}

/**
 * @description: 复制文字到剪贴板
 * @param {string} text
 * @return {*}
 */
export const copyText = (text: string) => {
  return new Promise((reslove, reject) => {
    navigator.clipboard
      .writeText(text)
      .then(() => {
        ElMessage({
          type: 'success',
          message: '复制成功',
        })
        reslove(true)
      })
      .catch(err => {
        ElMessage({
          type: 'error',
          message: '复制失败',
        })
        reject(err)
        throw new Error(err)
      })
  })
}

/**
 * @description: 用于清除reactive对象的所有属性并保持响应式
 * @param {Record} data 传入的对象
 * @return {*}
 */
export const clearReactiveData = (data: Record<string, any>) => {
  Object.keys(data).forEach(key => {
    delete data[key]
  })
}

/**
 * @description: 用于保存watch切换之前的数据
 * @param {any} data  需要保存的数据
 * @param {any} store 需要保存到的对象
 * @return {*}
 */
export const saveWatchData = (data: any, store: any) => {
  if (Array.isArray(data)) {
    // 这里需要深拷贝，否则会导致引用问题
    store.splice(0, store.length, ...JSON.parse(JSON.stringify(data)))
  } else {
    Object.assign(store, JSON.parse(JSON.stringify(data)))
  }
}

/**
 * @description: 用于比较watch切换之前的数据
 * @param {any} data 需要比较的数据
 * @param {any} store 需要比较到的对象
 * @return {*}  boolean
 */
export const compareWatchData = (data: any, store: any): boolean => {
  return JSON.stringify(data) === JSON.stringify(store)
}

/**
 * @description:   模糊查询
 * @param {string} pattern 需要匹配的模式，即正则表达式
 * @param {string} text   需要被搜索的目标字符串
 * @param {boolean} matchCase 是否区分大小写，默认为true，即不区分大小写
 * @return {boolean} 返回匹配结果
 */
export const fuzzySearch = (
  pattern: string,
  text: string,
  matchCase: boolean = true,
): boolean => {
  console.log('执行')
  console.log(pattern, text)
  const regex = new RegExp(pattern, matchCase ? 'i' : '') // 'i' 标志表示忽略大小写
  return regex.test(text)
}

/**
 * @description: 清空reactive对象的所有属性并保持响应式
 * @param {Reactive} obj 传入的对象
 * @return {*}
 */
export const resetReactive = (
  obj: Reactive<{ [key: string]: any } | Array<any>>,
) => {
  if (Array.isArray(obj)) {
    obj.splice(0, obj.length)
  } else {
    Object.keys(obj).forEach(key => {
      delete obj[key]
    })
  }
}

/**
 * @description: 生成唯一文件名
 * @param {string} extension 扩展名
 * @return {string} 唯一文件名
 */
export const generateUniqueFilename = (extension: string = 'xlsx'): string => {
  const timestamp = Date.now() // 获取当前时间戳（毫秒级）
  const randomStr = Math.random().toString(36).substring(2, 8) // 生成6位随机字符串
  const uniqueFilename = `${timestamp}_${randomStr}.${extension}`
  return uniqueFilename
}

/**
 * 根据传入的日期字符串，返回对应的时间戳（秒级）
 * 如：Fri Feb 14 2025 17:06:14 GMT+0800 (中国标准时间)
 * @param dateString
 */
export const convertToTimestampInSeconds = (dateString: Date): number => {
  return Math.floor(new Date(dateString).getTime() / 1000)
}
