import type {
  BaseFieldItem,
  CustomIndicatorScheme,
  SaveCustomIncicatorScheme,
  SaveFields,
  TableData,
  TableFields,
  TableInfoItem,
} from '@/types/Tables/table'

class TableCustomIndicatorController {
  #tableName: string = '' // 整个表格的表格名
  #schemes: SaveCustomIncicatorScheme = {} // 存储所有自定义指标方案

  constructor(
    tableName: string,
    tableInfo: TableInfoItem<TableData, BaseFieldItem<TableFields>>,
  ) {
    this.#tableName = tableName
    let schemes = localStorage.getItem(this.#tableName)
    if (schemes) this.#schemes = JSON.parse(schemes)
    else {
      this.saveCustomScheme(JSON.parse(JSON.stringify(tableInfo)))
      // this.#schemes = {
      //   [this.getDeterministicKey('默认')]: this.#defaultScheme,
      // }
    }
  }

  /**
   * @description: 生成一个随机键名
   * @param {string} input 需要生成键名的字符串
   * @return {string} 生成的键名
   */
  getDeterministicKey(input: string): string {
    let hash = 2166136261 // FNV-1a 初始值
    for (let i = 0; i < input.length; i++) {
      hash ^= input.charCodeAt(i)
      hash = Math.imul(hash, 16777619) // 使用32位整数乘法
    }

    // 转换为十六进制字符串
    return (hash >>> 0).toString(16)
  }

  /**
   * @description: 保存新方案
   * @param {string} schemeName 用户的自定义方案名，如果没输入就是默认
   * @param {TableInfoItem} tableInfo 表格信息
   * @return {*}
   */
  saveCustomScheme(
    tableInfo: TableInfoItem<TableData, BaseFieldItem<TableFields>>,
    schemeName: string = '默认',
  ) {
    try {
      if (!schemeName) schemeName = '默认'
      tableInfo = JSON.parse(JSON.stringify(tableInfo))
      let keyName = this.getDeterministicKey(schemeName)
      // 新的方案
      let newScheme: SaveFields = {
        name: keyName, // 这个是拿来作为标识符，也同时是这个方案的键
        label: schemeName, // 方案名
        fields: tableInfo.fields, // 分组后的字段信息
        tableSortedFields: tableInfo.tableSortedFields, // 排序过后的表格字段，仅用于展示表格字段
      }
      // console.log('执行')

      this.#schemes[keyName] = newScheme

      localStorage.setItem(this.#tableName, JSON.stringify(this.#schemes))
    } catch (err) {
      console.error(err)
      ElMessage.error('保存失败')
    }
  }

  /**
   * @description: 获取自定义方案
   * @param {string} schemeName 方案名，不传则是直接返回所有方案，传了就是具体的某一种方案
   * @return {SaveFields} 方案
   */
  getCustomScheme(schemeName: string): SaveFields | undefined
  getCustomScheme(): SaveCustomIncicatorScheme | undefined
  getCustomScheme(
    schemeName?: string,
  ): SaveFields | SaveCustomIncicatorScheme | undefined {
    if (!schemeName)
      return Object.keys(this.#schemes).length > 0 ? this.#schemes : undefined
    let keyName = this.getDeterministicKey(schemeName)
    return JSON.parse(JSON.stringify(this.#schemes[keyName]))
  }

  /**
   * @description: 获取所有的自定义方案列表
   * @return {Array<CustomIndicatorScheme>} 所有的自定义方案
   */
  getSchemeList(): Array<CustomIndicatorScheme> {
    let info = this.getCustomScheme()
    let result: Array<CustomIndicatorScheme> = []
    if (!info) return []
    for (let [key, scheme] of Object.entries(info)) {
      result.push({
        name: key,
        label: scheme.label,
      })
    }
    return JSON.parse(JSON.stringify(result))
  }
}

export default TableCustomIndicatorController
